@@ -2,8 +2,8 @@
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """Lightweight :term:`computation` graphs for Python."""
 
-__version__ = "7.0.0"
-__release_date__ = "28 Apr 2020, 18:57"
+__version__ = "7.1.1"
+__release_date__ = "5 May 2020, 01:30"
 __title__ = "graphtik"
 __summary__ = __doc__.splitlines()[0]
 __license__ = "Apache-2.0"
@@ -42,7 +42,14 @@ from .config import (
 ## SEE ALSO: `.plot.active_plotter_plugged()`, `.plot.set_active_plotter()` &
 #  `.plot.get_active_plotter()` configs, not imported, unless plot is needed..
 
-from .modifiers import *  # noqa, on purpose to include any new modifiers
+from .modifiers import (
+    mapped,
+    optional,
+    vararg,
+    varargs,
+    sideffect,
+    sideffected,
+)
 from .netop import compose
 from .network import AbortedException, IncompleteExecutionError
-from .op import operation
+from .op import operation, NULL_OP
