@@ -15,6 +15,7 @@ Generic utilities, or :mod:`.plot` API but without polluting imports.
 import abc
 import inspect
 import logging
+from collections import abc as cabc
 from collections import defaultdict, namedtuple
 from functools import partial, partialmethod, wraps
 from typing import Any, Collection, List, Mapping, NamedTuple, Optional, Tuple, Union
@@ -72,7 +73,9 @@ class Token(str):
 
 #: When an operation function returns this special value,
 #: it implies operation has no result at all,
-#: (otherwise, it would have been a single result, ``None``).`
+#: (otherwise, it would have been a single result, ``None``).
+#: Usefull for :term:`partial outputs` who want to cancel their single result
+#: witout being delcared as :term:`returns dictionary`.
 NO_RESULT = Token("NO_RESULT")
 UNSET = Token("UNSET")
 
@@ -110,7 +113,9 @@ def astuple(i, argname, allowed_types=tuple):
     return i
 
 
-def func_name(fn, default=..., mod=None, fqdn=None, human=None) -> Optional[str]:
+def func_name(
+    fn, default=..., mod=None, fqdn=None, human=None, partials=None
+) -> Optional[str]:
     """
     FQDN of `fn`, descending into partials to print their args.
 
@@ -124,8 +129,10 @@ def func_name(fn, default=..., mod=None, fqdn=None, human=None) -> Optional[str]
         and locals, respectively (:pep:`3155`).
         *Sphinx* uses `fqdn=True` for generating IDs.
     :param human:
-        when true, partials denote their args like ``fn({"a": 1}, ...)`` in the returned text,
-        otherwise, just the (fqd-)name, appropriate for IDs.
+        when true, explain built-ins, and assume ``partials=True`` (if that was None)
+    :param partials:
+        when true (or omitted & `human` true), partials denote their args
+        like ``fn({"a": 1}, ...)``
 
     :return:
         a (possibly dot-separated) string, or `default` (unless this is ``...```).
@@ -160,11 +167,13 @@ def func_name(fn, default=..., mod=None, fqdn=None, human=None) -> Optional[str]
 
     TBD
     """
+    if partials is None:
+        partials = human
 
     if isinstance(fn, (partial, partialmethod)):
         # Always bubble-up errors.
         fn_name = func_name(fn.func, default, mod, fqdn, human)
-        if human:
+        if partials:
             args = ", ".join(str(i) for i in fn.args)
             kw = ", ".join(f"{k}={v}" for k, v in fn.keywords.items())
             args_str = ", ".join(i for i in (args, kw) + ("...",) if i)
@@ -377,11 +386,19 @@ class PlotArgs(NamedTuple):
     outputs: Collection = None
     #: Contains the computed results, which might be different from :attr:`plottable`.
     solution: "graphtik.network.Solution" = None
-    #: a mapping of nodes to cluster names
+    #: Either a mapping of node-names to dot(``.``)-separated cluster-names, or
+    #: false/true to enable :term:`plotter`'s default clustering of nodes based
+    #: on their dot-separated name parts.
+    #:
+    #: Note that if it's `None` (default), the plotter will cluster based on node-names,
+    #: BUT the Plan may replace the None with a dictionary with the "pruned" cluster
+    #: (when its :term:`dag` differs from network's :term:`graph`);
+    #: to suppress the pruned-cluster, pass a truthy, NON-dictionary value.
     clusters: Mapping = None
     #: If given, overrides :active plotter`.
     plotter: "graphtik.plot.Plotter" = None
     #: If given, overrides :term:`plot theme` plotter will use.
+    #: It can be any mapping, in which case it overrite the :term:`current theme`.
     theme: "graphtik.plot.Theme" = None
 
     #######
@@ -494,8 +511,8 @@ class Plottable(abc.ABC):
 
             :seealso: :attr:`.PlotArgs.plotter`
         :param theme:
-            Any :term:`plot theme` overrides; if none, the :attr:`.Plotter.default_theme`
-            of the :term:`active plotter` is used.
+            Any :term:`plot theme` or dictionary overrides; if none,
+            the :attr:`.Plotter.default_theme` of the :term:`active plotter` is used.
 
             :seealso: :attr:`.PlotArgs.theme`
         :param name:
@@ -508,28 +525,22 @@ class Plottable(abc.ABC):
             (optional) A :class:`nx.Digraph` with overrides to merge with the graph provided
             by underlying plottables (translated by the :term:`active plotter`).
 
-            It may contain "public" or "private *graph*, *node* & *edge* attributes:
+            It may contain *graph*, *node* & *edge* attributes for any usage,
+            but these conventions apply:
 
-            - "private" attributes: those starting with underscore(``_``),
-              handled by :term:`plotter`:
+            ``'graphviz.xxx'`` *(graph/node/edge attributes)*
+                Any "user-overrides" with this prefix are sent verbatim a `Graphviz`_
+                attributes.
 
-              - ``_fn_link_target`` & ``_fn_link_target`` *(node)*: if truthy,
-                override result 2-tuple of :meth:`_get_fn_link()`.
-              - ``_op_tooltip`` & ``_fn_tooltip`` *(node)*: if truthy,
-                override those derrived from :meth:`_make_op_tooltip()` &
-                :meth:`_make_op_tooltip()`.
-              - ``_no_plot``: nodes and/or edges skipped from plotting
+                .. Note::
+
+                    Remember to escape those values as `Graphviz`_ HTML-Like strings
+                    (use :func:`.plot.graphviz_html_string()`).
+
+            ``no_plot`` *(node/edge attribute)*
+                element skipped from plotting
                 (see *"Examples:"* section, below)
 
-            - "public" attributes: reaching `Graphviz`_ as-is, e.g.
-              to set ``spline -> ortho`` on the graph attributes
-              (this can also be achieved by modified :term:`plot theme`).
-
-              .. Note::
-
-                Remember to escape those values as `Graphviz`_ HTML-Like strings
-                (use :func:`.plot.graphviz_html_string()`).
-
             :seealso: :attr:`.PlotArgs.graph`
         :param inputs:
             an optional name list, any nodes in there are plotted
@@ -550,7 +561,22 @@ class Plottable(abc.ABC):
 
             :seealso: :attr:`.PlotArgs.solution`
         :param clusters:
-            an optional mapping of nodes --> cluster-names, to group them
+            Either a mapping, or false/true to enable :term:`plotter`'s default
+            clustering of nodes base on their dot-separated name parts.
+
+            Note that if it's `None` (default), the plotter will cluster based on node-names,
+            BUT the Plan may replace the None with a dictionary with the "pruned" cluster
+            (when its :term:`dag` differs from network's :term:`graph`);
+            to suppress the pruned-cluster, pass a truthy, NON-dictionary value.
+
+            Practically, when it is a:
+
+            - dictionary of node-names --> dot(``.``)-separated cluster-names,
+              it is respected, even if empty;
+            - truthy: cluster based on dot(``.``)-separated node-name parts;
+            - falsy: don't cluster at all.
+
+            :seealso: :attr:`.PlotArgs.clusters`
         :param jupyter_render:
             a nested dictionary controlling the rendering of graph-plots in Jupyter cells,
             if `None`, defaults to :data:`jupyter_render`; you may modify it in place
@@ -636,7 +662,7 @@ class Plottable(abc.ABC):
         digraph netop {
         fontname=italic;
         label=<netop>;
-        <a> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+        <a> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
         ...
 
         .. graphtik::
@@ -650,7 +676,7 @@ class Plottable(abc.ABC):
 
         >>> g = nx.DiGraph()  # the overlay
         >>> to_hide = netop.net.find_op_by_name("sub")
-        >>> g.add_node(to_hide, _no_plot=True)
+        >>> g.add_node(to_hide, no_plot=True)
         >>> dot = netop.plot(graph=g)
         >>> assert "<sub>" not in str(dot), str(dot)
 
@@ -673,11 +699,21 @@ class Plottable(abc.ABC):
 
         plot_args = PlotArgs(**kw)
 
-        from .plot import Plotter, get_active_plotter
+        from .plot import Plotter, Theme, get_active_plotter
 
+        ## Ensure a valid plotter in the args asap.
+        #
         if plotter and not isinstance(plotter, Plotter):
-            raise ValueError(f"Invalid `plotter` argument given: {plotter}")
-        plot_args = plot_args._replace(plotter=plotter or get_active_plotter())
+            raise TypeError(f"Invalid `plotter` argument given: {plotter}")
+        if not plotter:
+            plotter = get_active_plotter()
+        plot_args = plot_args._replace(plotter=plotter)
+
+        ## Overwrite any dictionaries over active theme asap.
+        #
+        if isinstance(theme, cabc.Mapping):
+            theme = plotter.default_theme.withset(**theme)
+            plot_args = plot_args._replace(theme=theme)
 
         plot_args = self.prepare_plot_args(plot_args)
         assert plot_args.graph, plot_args
