@@ -14,7 +14,7 @@ from .base import UNSET, Items, PlotArgs, Plottable, aslist, astuple, jetsam
 from .config import is_debug, reset_abort
 from .modifiers import optional, sideffect
 from .network import ExecutionPlan, Network, NodePredicate, Solution, yield_ops
-from .op import FunctionalOperation, Operation, reparse_operation_data
+from .op import FunctionalOperation, NULL_OP, Operation, reparse_operation_data
 
 log = logging.getLogger(__name__)
 
@@ -77,11 +77,10 @@ def _make_network(
     merge_set = iset()  # Preseve given node order.
     for op in operations:
         if isinstance(op, NetworkOperation):
-            merge_set.update(
-                proc_op(s, op.name) for s in op.net.graph if isinstance(s, Operation)
-            )
+            merge_set.update(proc_op(s, op.name) for s in yield_ops(op.net.graph))
         else:
             merge_set.add(proc_op(op))
+    merge_set = iset(i for i in merge_set if not isinstance(i, NULL_OP))
 
     assert all(bool(n) for n in merge_set)
     net = Network(*merge_set)
@@ -259,7 +258,9 @@ class NetworkOperation(Operation, Plottable):
         plot_args = plottable.prepare_plot_args(plot_args)
         assert plot_args.graph, plot_args
 
-        plot_args.graph.graph.setdefault("label", graphviz_html_string(self.name))
+        plot_args.graph.graph.setdefault(
+            "graphviz.label", graphviz_html_string(self.name)
+        )
         plot_args = plot_args._replace(plottable=self)
 
         return plot_args
@@ -307,7 +308,7 @@ class NetworkOperation(Operation, Plottable):
 
     def compute(
         self,
-        named_inputs: Mapping,
+        named_inputs: Mapping = UNSET,
         outputs: Items = UNSET,
         predicate: NodePredicate = UNSET,
     ) -> Solution:
@@ -349,6 +350,9 @@ class NetworkOperation(Operation, Plottable):
         See also :meth:`.Operation.compute()`.
         """
         try:
+            if named_inputs is UNSET:
+                named_inputs = {}
+
             net = self.net  # jetsam
             outputs = self.outputs if outputs == UNSET else outputs
             predicate = self.predicate if predicate == UNSET else predicate
@@ -433,7 +437,7 @@ def compose(
     """
     operations = (op1,) + operations
     if not all(isinstance(op, Operation) for op in operations):
-        raise ValueError(f"Non-Operation instances given: {operations}")
+        raise TypeError(f"Non-Operation instances given: {operations}")
 
     return NetworkOperation(
         operations,
