@@ -10,7 +10,16 @@ import time
 from collections import ChainMap, abc, defaultdict, namedtuple
 from functools import partial
 from itertools import chain, count
-from typing import Any, Callable, Collection, List, Mapping, Optional, Tuple, Union
+from typing import (
+    Any,
+    Callable,
+    Collection,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+    Union,
+)
 
 import networkx as nx
 from boltons.setutils import IndexedSet as iset
@@ -25,9 +34,16 @@ from .config import (
     is_parallel_tasks,
     is_reschedule_operations,
     is_skip_evictions,
-    is_solid_true,
+    first_solid,
+)
+from .modifiers import (
+    is_mapped,
+    is_optional,
+    is_pure_sideffect,
+    is_sideffect,
+    is_sideffected,
+    optional,
 )
-from .modifiers import mapped, optional, sideffect, vararg, varargs
 from .op import Operation
 
 NodePredicate = Callable[[Any, Mapping], bool]
@@ -105,11 +121,7 @@ def _unsatisfied_operations(dag, inputs: Collection) -> List:
             else:
                 # It's ok not to dig into edge-data("optional") here,
                 # we care about all needs, including broken ones.
-                real_needs = set(
-                    n
-                    for n in node.needs
-                    if not isinstance(n, (optional, vararg, varargs))
-                )
+                real_needs = set(n for n in node.needs if not is_optional(n))
                 if real_needs.issubset(op_satisfaction[node]):
                     # We have a satisfied operation; mark its output-data
                     # as ok.
@@ -131,25 +143,6 @@ def _unsatisfied_operations(dag, inputs: Collection) -> List:
 class Solution(ChainMap, Plottable):
     """
     A chain-map collecting :term:`solution` outputs and execution state (eg :term:`overwrites`)
-
-    .. attribute:: plan
-
-        the plan that produced this solution
-    .. attribute:: executed
-
-        A dictionary with keys the operations executed, and values their status:
-
-        - no key: not executed yet
-        - value None: execution ok
-        - value Exception: execution failed
-
-    .. attribute:: canceled
-
-        A sorted set of :term:`canceled operation`\\s due to upstream failures.
-    .. attribute:: finalized
-
-        a flag denoting that this instance cannot accept more results
-        (after the :meth:`finalized` has been invoked)
     """
 
     def __init__(self, plan, input_values: dict):
@@ -161,11 +154,22 @@ class Solution(ChainMap, Plottable):
         self._layers = {op: {} for op in yield_ops(reversed(plan.steps))}
         super().__init__(*self._layers.values(), input_values)
 
+        #: the plan that produced this solution
         self.plan = plan
+        #: A dictionary with keys the operations executed, and values their status:
+        #:
+        #: - no key:            not executed yet
+        #: - value None:        execution ok
+        #: - value Exception:   execution failed
+        #: - value Collection:  canceled provides
         self.executed = {}
+        #: A sorted set of :term:`canceled operation`\\s due to upstream failures.
         self.canceled = iset()  # not iterated, order not important, but ...
+        #: a flag controlled by `plan` (by invoking :meth:`finalized` is invoked)
+        #: which becomes `True` when this instance has finished accepting results.
         self.finalized = False
         self.elapsed_ms = {}
+        #: A unique identifier to distinguish separate flows in execution logs.
         self.solid = "%X" % random.randint(0, 2 ** 16)
 
         ## Cache context-var flags.
@@ -196,15 +200,37 @@ class Solution(ChainMap, Plottable):
         return clone
 
     def __repr__(self):
-        items = ", ".join(f"{k!r}: {v!r}" for k, v in self.items())
         if is_debug():
             return self.debugstr()
         else:
+            items = ", ".join(f"{k!r}: {v!r}" for k, v in self.items())
             return f"{{{items}}}"
 
     def debugstr(self):
-        # TODO: augment Solution.__repr__() when DEBUG-log enabled.
-        return f"{type(self).__name__}({dict(self)}, {self.plan})"
+        return f"{type(self).__name__}({dict(self)}, {self.plan}, {self.executed}, {self._layers})"
+
+    def _reschedule(self, dag, nbunch_to_break, op):
+        """Update dag/canceled/executed ops and return newly-canceled ops. """
+        dag.remove_edges_from(tuple(dag.out_edges(nbunch_to_break)))
+        canceled = _unsatisfied_operations(dag, self)
+        # Minus executed, bc partial-out op might not have any provides left.
+        newly_canceled = iset(canceled) - self.canceled - self.executed
+        self.canceled.update(newly_canceled)
+
+        if newly_canceled and log.isEnabledFor(logging.INFO):
+            reason = (
+                "failure of"
+                if isinstance(nbunch_to_break, Operation) > 1
+                else "rescheduled"
+            )
+            log.info(
+                "... (%s) CANCELING +%s ops%s due to %s of op(%s).",
+                self.solid,
+                len(newly_canceled),
+                [n.name for n in newly_canceled],
+                reason,
+                op.name,
+            )
 
     def operation_executed(self, op, outputs):
         """
@@ -217,38 +243,48 @@ class Solution(ChainMap, Plottable):
         :param op:
             the operation that completed ok
         :param outputs:
-            The names of the `outputs` values the op` actually produced,
+            The named values the op` actually produced,
             which may be a subset of its `provides`.  Sideffects are not considered.
 
         """
+
+        def collect_canceled_sideffects(dep, val) -> Collection:
+            """Return any sfx `dep` with falsy value, singularizing sideffected."""
+            if val or not is_sideffect(dep):
+                return ()
+            if is_pure_sideffect(dep):
+                return (dep,)
+            ## Singularize sideffected
+            assert is_sideffected(dep), locals()
+            return (dep.withset(sideffects=(s,)) for s in dep.sideffects)
+
         assert not self.finalized, f"Cannot reuse solution: {self}"
         self._layers[op].update(outputs)
         self.executed[op] = None
 
-        if is_solid_true(self.is_reschedule, op.rescheduled):
-            dag = self.dag
+        if first_solid(self.is_reschedule, op.rescheduled):
+            ## Find which provides have been broken?
+            #
+            # OPTIMIZE: could use _fn_provides
             missing_outs = iset(op.provides) - set(outputs)
-            to_brake = [
-                (op, out) for out in missing_outs if not isinstance(out, sideffect)
+            sfx = (out for out in missing_outs if is_sideffect(out))
+            canceled_sideffects = [
+                sf
+                for k, v in outputs.items()
+                for sf in collect_canceled_sideffects(k, v)
             ]
-            if to_brake:
-                self.executed[op] = list(
-                    missing_outs
-                )  # list checked by `scream_if_incomplete()`
-                dag.remove_edges_from(to_brake)
-                canceled = _unsatisfied_operations(dag, self)
-                # Minus executed, bc partial-out op might not have any provides left.
-                newly_canceled = iset(canceled) - self.canceled - self.executed
-                if newly_canceled and log.isEnabledFor(logging.INFO):
-                    log.info(
-                        "... (%s) SKIPPING +%s ops%s due to partial outs%s of op(%s).",
-                        self.solid,
-                        len(newly_canceled),
-                        [n.name for n in newly_canceled],
-                        list(missing_outs),
-                        op.name,
-                    )
-                self.canceled.update(newly_canceled)
+            to_break = (missing_outs - sfx) | canceled_sideffects
+            log.info(
+                "... (%s) missing partial outputs %s from rescheduled %s.",
+                self.solid,
+                list(to_break),
+                op,
+            )
+
+            if to_break:
+                self._reschedule(self.dag, to_break, op)
+                # list used by `check_if_incomplete()`
+                self.executed[op] = to_break
 
     def operation_failed(self, op, ex):
         """
@@ -259,20 +295,7 @@ class Solution(ChainMap, Plottable):
         """
         assert not self.finalized, f"Cannot reuse solution: {self}"
         self.executed[op] = ex
-
-        dag = self.dag
-        dag.remove_edges_from(list(dag.out_edges(op)))
-        canceled = _unsatisfied_operations(dag, self)
-        newly_canceled = iset(canceled) - self.canceled
-        if newly_canceled and log.isEnabledFor(logging.INFO):
-            log.info(
-                "... (%s) SKIPPING +%s ops%s due to failed op(%s).",
-                self.solid,
-                len(newly_canceled),
-                [n.name for n in newly_canceled],
-                op.name,
-            )
-        self.canceled.update(newly_canceled)
+        self._reschedule(self.dag, op, op)
 
     def finalize(self):
         """invoked only once, after all ops have been executed"""
@@ -315,9 +338,9 @@ class Solution(ChainMap, Plottable):
         if incomplete:
             incomplete = [op.name for op in incomplete]
             partial_msgs = {
-                f"\n  +--{op.name}: {pouts}"
+                f"\n  +--{op.name}: {list(pouts)}"
                 for op, pouts in self.executed.items()
-                if pouts and isinstance(pouts, list)
+                if pouts and isinstance(pouts, abc.Collection)
             }
             err_msgs = [
                 f"\n  +--{op.name}: {type(ex).__name__}('{ex}')"
@@ -383,7 +406,7 @@ def _optionalized(graph, data):
         optional(data)
         if all_optionals
         else data  # sideffect
-        if isinstance(data, sideffect)
+        if is_sideffect(data)
         else str(data)  # un-optionalize
     )
 
@@ -588,12 +611,12 @@ class ExecutionPlan(
                 solution.elapsed_ms[op] = time.time()
 
                 task = _OpTask(op, input_values, solution.solid)
-                if is_solid_true(global_marshal, op.marshalled):
+                if first_solid(global_marshal, op.marshalled):
                     task = task.marshalled()
 
-                if is_solid_true(global_parallel, op.parallel):
+                if first_solid(global_parallel, op.parallel):
                     if not pool:
-                        raise ValueError(
+                        raise RuntimeError(
                             "With `parallel` you must `set_execution_pool().`"
                         )
 
@@ -639,7 +662,7 @@ class ExecutionPlan(
                 "... (%s) op(%s) completed in %sms.", solution.solid, op.name, elapsed
             )
         except Exception as ex:
-            is_endured = solution.is_endurance or op.endured
+            is_endured = first_solid(solution.is_endurance, op.endured)
             elapsed = elapsed_ms(op)
             loglevel = logging.WARNING if is_endured else logging.DEBUG
             log.log(
@@ -725,6 +748,14 @@ class ExecutionPlan(
 
             # stop if no nodes left to schedule, exit out of the loop
             if not upnext:
+                ## Re-check evictions and assume all ops executed.
+                #  Sequenced executor has no such problem bc exhausts steps.
+                #
+                #  TODO: evictions for parallel are wasting loops.
+                #
+                for node in self.steps:
+                    if isinstance(node, _EvictInstruction) and node in solution:
+                        del solution[node]
                 break
 
             if _isDebugLogging():
@@ -854,14 +885,15 @@ class ExecutionPlan(
 
             # Validate eviction was perfect
             #
-            assert (
-                not evict
+            if evict:
+                expected_provides = set(
+                    n.sideffected if is_sideffected(n) else n for n in self.provides
+                )
                 # It is a proper subset when not all outputs calculated.
-                or set(solution).issubset(self.provides)
-            ), (
-                f"Evictions left more data{list(iset(solution) - set(self.provides))} than {self}!"
-                ' \n Did you bypass "impossible-outputs" validation?'
-            )
+                assert set(solution).issubset(expected_provides), (
+                    f"Evictions left more data{list(iset(solution) - set(self.provides))} than {self}!"
+                    ' \n Did you bypass "impossible-outputs" validation?'
+                )
 
             return solution
         except Exception as ex:
@@ -908,7 +940,7 @@ class Network(Plottable):
             graph = nx.DiGraph()
         else:
             if not isinstance(graph, nx.Graph):
-                raise ValueError(f"Must be a NetworkX graph, was: {graph}")
+                raise TypeError(f"Must be a NetworkX graph, was: {graph}")
 
         #: The :mod:`networkx` (Di)Graph containing all operations and dependencies,
         #: prior to :term:`compilation`.
@@ -982,12 +1014,12 @@ class Network(Plottable):
         needs_edges = []
         for n in getattr(operation, "op_needs", operation.needs):
             nkw, ekw = {}, {}
-            if isinstance(n, (optional, vararg, varargs)):
+            if is_optional(n):
                 ekw["optional"] = True
-            if isinstance(n, sideffect):
+            if is_sideffect(n):
                 ekw["sideffect"] = nkw["sideffect"] = True
-            if isinstance(n, mapped) and n.fn_arg is not None:
-                ekw["_fn_arg"] = n.fn_arg
+            if is_mapped(n):
+                ekw["fn_kwarg"] = n.fn_kwarg
             needs.append((n, nkw))
             needs_edges.append((n, operation, ekw))
         graph.add_nodes_from(needs)
@@ -1004,13 +1036,13 @@ class Network(Plottable):
         #
         for n in getattr(operation, "op_provides", operation.provides):
             kw = {}
-            if isinstance(n, sideffect):
+            if is_sideffect(n):
                 kw["sideffect"] = True
                 graph.add_node(n, sideffect=True)
 
             if n in alias_sources:
                 src_provide = alias_sources[n]
-                kw["_alias_of"] = src_provide
+                kw["alias_of"] = src_provide
 
             graph.add_edge(operation, n, **kw)
 
@@ -1150,7 +1182,9 @@ class Network(Plottable):
         )
         if outputs is None:
             outputs = iset(
-                n for n in self.provides if n not in inputs and n in pruned_dag
+                n
+                for n in self.provides
+                if n not in inputs and n in pruned_dag and not is_sideffect(n)
             )
         else:
             # filter-out from new `provides` if pruned.
