@@ -25,8 +25,24 @@ from .base import (
     func_name,
     jetsam,
 )
-from .config import is_debug, is_reschedule_operations, is_solid_true
-from .modifiers import mapped, optional, sideffect, sol_sideffect, vararg, varargs
+from .config import (
+    is_debug,
+    is_endure_operations,
+    is_marshal_tasks,
+    is_parallel_tasks,
+    is_reschedule_operations,
+    first_solid,
+)
+from .modifiers import (
+    is_mapped,
+    is_optional,
+    is_pure_sideffect,
+    is_sideffect,
+    is_sideffected,
+    is_vararg,
+    is_varargs,
+    sideffected,
+)
 
 log = logging.getLogger(__name__)
 
@@ -57,7 +73,7 @@ def as_renames(i, argname):
     if isinstance(i, tuple) and len(i) == 2:
         i = [i]
     elif not isinstance(i, cabc.Collection):
-        raise ValueError(
+        raise TypeError(
             f"Argument {argname} must be a list of 2-element items, was: {i!r}"
         ) from None
     elif not is_list_of_2(i):
@@ -83,17 +99,17 @@ def reparse_operation_data(
     """
 
     if not isinstance(name, cabc.Hashable):
-        raise ValueError(f"Operation `name` must be hashable, got: {name}")
+        raise TypeError(f"Operation `name` must be hashable, got: {name}")
 
     # Allow single string-value for needs parameter
     needs = astuple(needs, "needs", allowed_types=cabc.Collection)
     if not all(isinstance(i, str) for i in needs):
-        raise ValueError(f"All `needs` must be str, got: {needs!r}")
+        raise TypeError(f"All `needs` must be str, got: {needs!r}")
 
     # Allow single value for provides parameter
     provides = astuple(provides, "provides", allowed_types=cabc.Collection)
     if not all(isinstance(i, str) for i in provides):
-        raise ValueError(f"All `provides` must be str, got: {provides!r}")
+        raise TypeError(f"All `provides` must be str, got: {provides!r}")
 
     return name, needs, provides
 
@@ -130,26 +146,27 @@ def _spread_sideffects(
 
     :return:
         the given `deps` duplicated as ``(fn_deps,  op_deps)``, where any instances of
-        :class:`.sideffects` and :class:`.sol_sideffect` are processed like this:
+        :func:`.sideffect` and :func:`.sideffected` are processed like this:
 
         `fn_deps`
-            - :class:`.sol_sideffect` are replaced by the pure :attr:`.sol_sideffect.sideffected`
-              consumed/produced by underlying functions, in the order it is first met
-              (the rest duplicate `sideffected` are discarded).
-            - :class:`.sideffects` are simply dropped;
+            - :func:`.sideffected` are replaced by the pure :attr:`._Modifier.sideffected`
+              dependency consumed/produced by underlying functions, in the order
+              it is first met (the rest duplicate `sideffected` are discarded).
+            - :func:`.sideffect` are simply dropped;
 
         `op_deps`
-            :class:`.sol_sideffect` are replaced by a sequence of "singular" `sol_sideffect`
-            instances, one for each item in their :attr:`.sol_sideffect.sideffects` attribute,
+            :func:`.sideffected` are replaced by a sequence of "singular" `sideffected`
+            instances, one for each item in their :attr:`._Modifier.sideffects` attribute,
             in the order they are first met
             (any duplicates are discarded, order is irrelevant, since they don't reach
             the function);
     """
 
-    def singularize_sol_sideffects(dep):
+    def singularize_sideffecteds(dep):
+        """Make 1 sideffected for each pure-sfx contained, preserving attributes """
         return (
-            (sol_sideffect(dep.sideffected, s) for s in dep.sideffects)
-            if isinstance(dep, sol_sideffect)
+            (dep.withset(sideffects=(s,)) for s in dep.sideffects)
+            if is_sideffected(dep)
             else (dep,)
         )
 
@@ -158,19 +175,20 @@ def _spread_sideffects(
     seen_sideffecteds: Set[str] = set()
 
     def strip_sideffecteds(dep):
-        if isinstance(dep, sol_sideffect):
+        """Strip the dependency, preserving attributes """
+        if is_sideffected(dep):
             sideffected = dep.sideffected
             if not sideffected in seen_sideffecteds:
                 seen_sideffecteds.add(sideffected)
-                return (sideffected,)
-        elif not isinstance(dep, sideffect):
+                return (dep.withset(sideffects=None),)
+        elif not is_sideffect(dep):
             return (dep,)
         return ()
 
     assert deps is not None
 
     if deps:
-        deps = tuple(nn for n in deps for nn in singularize_sol_sideffects(n))
+        deps = tuple(nn for n in deps for nn in singularize_sideffecteds(n))
         fn_deps = tuple(nn for n in deps for nn in strip_sideffecteds(n))
         return deps, fn_deps
     else:
@@ -212,18 +230,18 @@ class FunctionalOperation(Operation, Plottable):
         node_props = node_props = node_props if node_props else {}
 
         if fn and not callable(fn):
-            raise ValueError(f"Operation was not provided with a callable: {fn}")
+            raise TypeError(f"Operation was not provided with a callable: {fn}")
         if parents and not isinstance(parents, tuple):
-            raise ValueError(
+            raise TypeError(
                 f"Operation `parents` must be tuple, was {type(parents).__name__!r}: {parents}"
             )
         if node_props is not None and not isinstance(node_props, cabc.Mapping):
-            raise ValueError(
+            raise TypeError(
                 f"Operation `node_props` must be a dict, was {type(node_props).__name__!r}: {node_props}"
             )
 
         if name is None and fn:
-            name = func_name(fn, None, mod=0, fqdn=0, human=0)
+            name = func_name(fn, None, mod=0, fqdn=0, human=0, partials=1)
         if name is not None:
             name = ".".join(str(pop) for pop in ((parents or ()) + (name,)))
         ## Overwrite reparsed op-data.
@@ -252,7 +270,7 @@ class FunctionalOperation(Operation, Plottable):
             sfx_aliases = [
                 f"{src} -> {dst}"
                 for src, dst in aliases
-                if isinstance(src, sideffect) or isinstance(dst, sideffect)
+                if is_sideffect(src) or is_sideffect(dst)
             ]
             if sfx_aliases:
                 raise ValueError(
@@ -263,32 +281,33 @@ class FunctionalOperation(Operation, Plottable):
             alias_dst = ()
         op_provides = iset(itt.chain(provides, alias_dst))
 
+        #: The :term:`operation`'s underlying function.
         self.fn = fn
         #: a name for the operation (e.g. `'conv1'`, `'sum'`, etc..);
         #: it will be prefixed by `parents`.
         self.name = name
 
         #: The :term:`needs` almost as given by the user
-        #: (which may contain MULTI-sol_sideffects and dupes),
+        #: (which may contain MULTI-sideffecteds and dupes),
         #: roughly morphed into `_fn_provides` + sideffects
-        #: (dupes preserved, with sideffects & SINGULARIZED :term:`solution sideffect`\s).
+        #: (dupes preserved, with sideffects & SINGULARIZED :term:`sideffected`\s).
         #: It is stored for builder functionality to work.
         self.needs = needs
         #: Value names ready to lay the graph for :term:`pruning`
-        #: (NO dupes, WITH aliases & sideffects, and SINGULAR :term:`solution sideffect`\s).
+        #: (NO dupes, WITH aliases & sideffects, and SINGULAR :term:`sideffected`\s).
         self.op_needs = op_needs
         #: Value names the underlying function requires
         #: (dupes preserved, without sideffects, with stripped :term:`sideffected` dependencies).
         self._fn_needs = _fn_needs
 
         #: The :term:`provides` almost as given by the user
-        #: (which may contain MULTI-sol_sideffects and dupes),
+        #: (which may contain MULTI-sideffecteds and dupes),
         #: roughly morphed into `_fn_provides` + sideffects
-        #: (dupes preserved, without aliases, with sideffects & SINGULARIZED :term:`solution sideffect`\s).
+        #: (dupes preserved, without aliases, with sideffects & SINGULARIZED :term:`sideffected`\s).
         #: It is stored for builder functionality to work.
         self.provides = provides
         #: Value names ready to lay the graph for :term:`pruning`
-        #: (NO dupes, WITH aliases & sideffects, and SINGULAR sol_sideffects).
+        #: (NO dupes, WITH aliases & sideffects, and SINGULAR sideffecteds).
         self.op_provides = op_provides
         #: Value names the underlying function produces
         #: (dupes preserved, without aliases & sideffects, with stripped :term:`sideffected` dependencies).
@@ -315,9 +334,14 @@ class FunctionalOperation(Operation, Plottable):
         #: along with its `inputs` & `outputs`.
         #: (usefull when run in `parallel` with a :term:`process pool`).
         self.marshalled = marshalled
-        #: if true, it means the `fn` returns a dictionary with all `provides`,
-        #: and no further processing is done on them
-        #: (i.e. the returned output-values are not zipped with `provides`)
+        #: If true, it means the underlying function :term:`returns dictionary` ,
+        #: and no further processing is done on its results,
+        #: i.e. the returned output-values are not zipped with `provides`.
+        #:
+        #: It does not have to return any :term:`alias` `outputs`.
+        #:
+        #: Can be changed amidst execution by the operation's function,
+        #: but it is easier for that function to simply call :meth:`.set_results_by_name()`.
         self.returns_dict = returns_dict
         #: Added as-is into NetworkX graph, and you may filter operations by
         #: :meth:`.NetworkOperation.withset()`.
@@ -345,12 +369,14 @@ class FunctionalOperation(Operation, Plottable):
         aliases = aslist(self.aliases, "aliases")
         aliases = f", aliases={aliases!r}" if aliases else ""
         fn_name = self.fn and func_name(self.fn, None, mod=0, fqdn=0, human=0)
-        returns_dict_marker = self.returns_dict and "{}" or ""
         nprops = f", x{len(self.node_props)}props" if self.node_props else ""
-        resched = "?" if self.rescheduled else ""
-        endured = "!" if self.endured else ""
-        parallel = "|" if self.parallel else ""
-        marshalled = "$" if self.marshalled else ""
+        resched = (
+            "?" if first_solid(self.rescheduled, is_reschedule_operations()) else ""
+        )
+        endured = "!" if first_solid(self.endured, is_endure_operations()) else ""
+        parallel = "|" if first_solid(self.parallel, is_parallel_tasks()) else ""
+        marshalled = "&" if first_solid(self.marshalled, is_marshal_tasks()) else ""
+        returns_dict_marker = self.returns_dict and "{}" or ""
 
         if is_debug():
             debug_needs = (
@@ -429,22 +455,64 @@ class FunctionalOperation(Operation, Plottable):
         msg = textwrap.indent("\n".join(errors), " " * 4)
         raise MultiValueError(f"Failed preparing needs: \n{msg}", *exceptions)
 
-    def _zip_results_with_provides(self, results, fn_expected: iset) -> dict:
-        """Zip results with expected "real" (without sideffects) `provides`."""
-        rescheduled = is_solid_true(is_reschedule_operations(), self.rescheduled)
-        if not fn_expected:  # All provides were sideffects?
-            if results and results != NO_RESULT:
-                ## Do not scream,
-                #  it is common to call a function for its sideffects,
-                # which happens to return an irrelevant value.
-                log.warning(
-                    "Ignoring result(%s) because no `provides` given!\n  %s",
-                    results,
-                    self,
-                )
-            results = {}
+    def _match_inputs_with_fn_needs(self, named_inputs) -> Tuple[list, list, dict]:
+        positional, vararg_vals = [], []
+        kwargs = {}
+        errors, missing, varargs_bad = [], [], []
+        for n in self._fn_needs:
+            assert not is_sideffect(n), locals()
+            try:
+                if n not in named_inputs:
+                    if not is_optional(n) or is_sideffect(n):
+                        # It means `inputs` < compulsory `needs`.
+                        # Compilation should have ensured all compulsories existed,
+                        # but ..?
+                        ##
+                        missing.append(n)
+                    continue
 
-        elif self.returns_dict:
+                ## TODO: augment modifiers with "retrievers" from `inputs`.
+                inp_value = named_inputs[n]
+
+                if is_mapped(n):
+                    kwargs[n.fn_kwarg] = inp_value
+
+                elif is_vararg(n):
+                    vararg_vals.append(inp_value)
+
+                elif is_varargs(n):
+                    if isinstance(inp_value, str) or not isinstance(
+                        inp_value, cabc.Iterable
+                    ):
+                        varargs_bad.append(n)
+                    else:
+                        vararg_vals.extend(i for i in inp_value)
+
+                else:
+                    positional.append(inp_value)
+
+            except Exception as nex:
+                log.debug(
+                    "Cannot prepare op(%s) need(%s) due to: %s",
+                    self.name,
+                    n,
+                    nex,
+                    exc_info=nex,
+                )
+                errors.append((n, nex))
+
+        if errors or missing or varargs_bad:
+            raise self._prepare_match_inputs_error(
+                errors, missing, varargs_bad, named_inputs
+            )
+
+        return positional, vararg_vals, kwargs
+
+    def _zip_results_with_provides(self, results) -> dict:
+        """Zip results with expected "real" (without sideffects) `provides`."""
+        fn_expected: iset = self._fn_provides
+        rescheduled = first_solid(is_reschedule_operations(), self.rescheduled)
+        if self.returns_dict:
 
             if hasattr(results, "_asdict"):  # named tuple
                 results = results._asdict()
@@ -458,21 +526,30 @@ class FunctionalOperation(Operation, Plottable):
                     f"got {type(results).__name__!r}: {results}\n  {self}"
                 )
 
+            if rescheduled:
+                fn_required = fn_expected
+                # Canceled sfx are welcomed.
+                fn_expected = iset(self.provides)
+            else:
+                fn_required = fn_expected
+
             res_names = results.keys()
 
             ## Allow unknown outs when dict,
             #  bc we can safely ignore them (and it's handy for reuse).
             #
-            if res_names - fn_expected:
-                unknown = list(res_names - fn_expected)
+            unknown = res_names - fn_expected
+            if unknown:
+                unknown = list(unknown)
                 log.info(
-                    "Results%s contained +%s unknown provides%s\n  {self}",
+                    "Results%s contained +%s unknown provides%s\n  %s",
                     list(res_names),
                     len(unknown),
                     list(unknown),
+                    self,
                 )
 
-            missmatched = fn_expected - res_names
+            missmatched = fn_required - res_names
             if missmatched:
                 if rescheduled:
                     log.warning(
@@ -486,6 +563,22 @@ class FunctionalOperation(Operation, Plottable):
                         f"-{len(missmatched)} provides({list(fn_expected)})!\n  {self}"
                     )
 
+        elif results == NO_RESULT and rescheduled:
+            # Cancel also any SFX.
+            results = {p: False for p in set(self.provides) if is_sideffect(p)}
+
+        elif not fn_expected:  # All provides were sideffects?
+            if results and results != NO_RESULT:
+                ## Do not scream,
+                #  it is common to call a function for its sideffects,
+                # which happens to return an irrelevant value.
+                log.warning(
+                    "Ignoring result(%s) because no `provides` given!\n  %s",
+                    results,
+                    self,
+                )
+            results = {}
+
         else:  # Handle result sequence: no-result, single-item, many
             nexpected = len(fn_expected)
 
@@ -503,7 +596,7 @@ class FunctionalOperation(Operation, Plottable):
                 if isinstance(results, (str, bytes)) or not isinstance(
                     results, cabc.Iterable
                 ):
-                    raise ValueError(
+                    raise TypeError(
                         f"Expected x{nexpected} ITERABLE results, "
                         f"got {type(results).__name__!r}: {results}\n  {self}"
                     )
@@ -546,73 +639,28 @@ class FunctionalOperation(Operation, Plottable):
 
         return results
 
-    def compute(self, named_inputs, outputs=None) -> dict:
+    def compute(self, named_inputs=None, outputs: Items = None) -> dict:
         try:
             if self.fn is None:
                 raise ValueError(
                     f"Operation was not yet provided with a callable `fn`!"
                 )
             assert self.name is not None, self
+            if named_inputs is None:
+                named_inputs = {}
 
-            positional, vararg_vals = [], []
-            kwargs = {}
-            errors, missing, varargs_bad = [], [], []
-            for n in self._fn_needs:
-                assert not isinstance(n, sideffect), locals()
-                try:
-                    if n not in named_inputs:
-                        if not isinstance(n, (optional, vararg, varargs, sideffect)):
-                            # It means `inputs` < compulsory `needs`.
-                            # Compilation should have ensured all compulsories existed,
-                            # but ..?
-                            ##
-                            missing.append(n)
-                        continue
+            positional, varargs, kwargs = self._match_inputs_with_fn_needs(named_inputs)
+            results_fn = self.fn(*positional, *varargs, **kwargs)
+            results_op = self._zip_results_with_provides(results_fn)
 
-                    ## TODO: augment modifiers with "retrievers" from `inputs`.
-                    inp_value = named_inputs[n]
-
-                    if isinstance(n, mapped):  # includes `optionals`
-                        kwargs[n if n.fn_arg is None else n.fn_arg] = inp_value
-
-                    elif isinstance(n, vararg):
-                        vararg_vals.append(inp_value)
-
-                    elif isinstance(n, varargs):
-                        if isinstance(inp_value, str) or not isinstance(
-                            inp_value, cabc.Iterable
-                        ):
-                            varargs_bad.append(n)
-                        else:
-                            vararg_vals.extend(i for i in inp_value)
-
-                    else:
-                        positional.append(inp_value)
-
-                except Exception as nex:
-                    log.debug(
-                        "Cannot prepare op(%s) need(%s) due to: %s",
-                        self.name,
-                        n,
-                        nex,
-                        exc_info=nex,
-                    )
-                    errors.append((n, nex))
-
-            if errors or missing or varargs_bad:
-                raise self._prepare_match_inputs_error(
-                    errors, missing, varargs_bad, named_inputs
-                )
-
-            results_fn = self.fn(*positional, *vararg_vals, **kwargs)
-
-            # TODO: rename op jetsam (real_)provides --> fn_expected
-            provides = self._fn_provides
-            results_op = self._zip_results_with_provides(results_fn, provides)
+            outputs = astuple(outputs, "outputs", allowed_types=cabc.Collection)
 
+            ## Keep only outputs asked.
+            #  Note that plan's executors do not ask outputs
+            #  (see `_OpTask.__call__`).
+            #
             if outputs:
-                outputs = set(n for n in outputs if not isinstance(n, sideffect))
-                # Ignore sideffect outputs.
+                outputs = set(n for n in outputs)
                 results_op = {
                     key: val for key, val in results_op.items() if key in outputs
                 }
@@ -624,32 +672,27 @@ class FunctionalOperation(Operation, Plottable):
                 locals(),
                 "outputs",
                 "aliases",
-                "provides",
                 "results_fn",
                 "results_op",
                 operation="self",
                 args=lambda locs: {
                     "positional": locs.get("positional"),
-                    "varargs": locs.get("vararg_vals"),
+                    "varargs": locs.get("varargs"),
                     "kwargs": locs.get("kwargs"),
                 },
             )
             raise
 
     def __call__(self, *args, **kwargs):
-        """
-        Although may return results like :meth:`compute()`, does no checks, and
-
-        it it passes args/kw as user desires.
-        """
-        return self.fn(*args, **kwargs)
+        """Like dict args, delegates to :meth:`.compute()`."""
+        return self.compute(dict(*args, **kwargs))
 
     def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
         """Delegate to a provisional network with a single op . """
         from .netop import compose
         from .plot import graphviz_html_string
 
-        is_user_label = bool(plot_args.graph and plot_args.graph.get("label"))
+        is_user_label = bool(plot_args.graph and plot_args.graph.get("graphviz.label"))
         plottable = compose(self.name, self)
         plot_args = plot_args.with_defaults(name=self.name)
         plot_args = plottable.prepare_plot_args(plot_args)
@@ -657,8 +700,8 @@ class FunctionalOperation(Operation, Plottable):
 
         ## Operations don't need another name visible.
         #
-        if not is_user_label:
-            del plot_args.graph.graph["label"]
+        if is_user_label:
+            del plot_args.graph.graph["graphviz.label"]
         plot_args = plot_args._replace(plottable=self)
 
         return plot_args
@@ -814,7 +857,18 @@ def operation(
     def decorator(*args, **kw):
         return op.withset(*args, **kw)
 
-    # Allow the decorator to support the builder-pattern.
+    # Dress decorator to support the builder-pattern.
+    # even when called as regular function (without `@`) without `fn`.
     decorator.withset = op.withset
 
     return decorator
+
+
+class NULL_OP(FunctionalOperation):
+    """Eliminates same-named operations added later during term:`operation merging`."""
+
+    def __init__(self, name, parents=None):
+        super().__init__(name=name, parents=parents)
+
+    def compute(self, *args, **kw):
+        raise AssertionError("Should have been eliminated!")
