@@ -266,6 +266,15 @@ class _GraphtikTestDirective(extdoctest.TestDirective):
 
         node = graphtik_node(graphvar=options.get("graphvar"), img_format=img_format)
         node.source, node.line = location
+        ## Decide a unique filename (and id).
+        #
+        name = options.get("name") or ""
+        if name:
+            name = nodes.fully_normalize_name(name)
+        targetname = (
+            f"graphtik-{self.env.docname}-{name}-{self.env.new_serialno('graphtik')}"
+        )
+        node["filename"] = targetname
         node += original_nodes
 
         figure = nodes.figure()
@@ -277,8 +286,8 @@ class _GraphtikTestDirective(extdoctest.TestDirective):
         figure["classes"].extend(options.get("figclass", "").split())
         node += figure
 
-        img_attrs = {k: v for k, v in options.items() if k in _img_options}
         # TODO: emulate sphinx-processing for image width & height attrs.
+        img_attrs = {k: v for k, v in options.items() if k in _img_options}
         image = dynaimage(**img_attrs)
         image.source, image.line = location
         image["classes"].extend(options.get("class", "").split())
@@ -309,15 +318,27 @@ class _GraphtikTestDirective(extdoctest.TestDirective):
             #
             name = options.pop("name")
             if not caption:
-                caption = _("plottable: ``%s``") % name
-            name = nodes.fully_normalize_name(name)
-            targetname = f"graphtik-{name}"
+                caption = name
             figure["names"].append(targetname)
-            self.state.document.note_explicit_target(figure, figure)
             ## adapted from: sphinx.domains.std.Target directive.
             #
             std = cast(StandardDomain, self.env.get_domain("std"))
-            std.add_object("graphtik", name, self.env.docname, targetname)
+            ## Suppress warning on duplicates (replaced with INFO).
+            #  Duplicates occur either by include directives
+            #  or because :noindex: in autoclass is ignored here.
+            #
+            objtype = "graphtik"
+            if (objtype, name) in std.objects:
+                docname = std.objects[objtype, name][0]
+                log.info(
+                    __("Skipping duplicate %s description of %s, other instance in %s"),
+                    objtype,
+                    name,
+                    docname,
+                    location=location,
+                )
+            else:
+                std.note_object(objtype, name, targetname, figure)
 
         ## See sphinx.ext.graphviz:figure_wrapper(),
         #  and <sphinx.git>/tests/roots/test-add_enumerable_node/enumerable_node.py:MyFigure
@@ -418,7 +439,7 @@ class DocFilesPurgatory:
                     log.debug(
                         "Deleting outdated image '%s' of doc %r...", docname, fpath
                     )
-                    fpath.unlink(missing_ok=True)
+                    fpath.unlink()
                 except Exception as ex:
                     log.warning(
                         "Ignoring error while deleting outdated fpath '%s': %s",
