@@ -152,7 +152,7 @@ class _ScreamingOperation(op.Operation):
                 operation(name="test", needs=["a"], provides=["b"])(_scream).compute,
                 named_inputs={"a": 1},
             ),
-            "outputs provides aliases results_fn results_op operation args".split(),
+            "outputs aliases results_fn results_op operation args".split(),
         ),
         (
             lambda: fnt.partial(
@@ -185,7 +185,7 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
             lambda: fnt.partial(
                 operation(_scream, name="test").compute, named_inputs=None
             ),
-            "outputs provides aliases results_fn results_op operation args".split(),
+            "outputs aliases results_fn results_op operation args".split(),
         ),
         (
             lambda: fnt.partial(
@@ -436,6 +436,22 @@ def test_func_name_lambda_local(kw, exp):
     assert base.func_name(lambda: None, **kw) == exp
 
 
+def test_func_name_partials_vs_human():
+    assert base.func_name(fnt.partialmethod(_Foo.foo), human=1, partials=0) == "foo"
+    assert base.func_name(fnt.partialmethod(_Foo.foo), human=None, partials=0) == "foo"
+    assert (
+        base.func_name(fnt.partialmethod(_Foo.foo), human=1, partials=1) == "foo(...)"
+    )
+    assert (
+        base.func_name(fnt.partialmethod(_Foo.foo), human=None, partials=1)
+        == "foo(...)"
+    )
+    assert (
+        base.func_name(fnt.partialmethod(_Foo.foo), human=1, partials=None)
+        == "foo(...)"
+    )
+
+
 ###############
 ## func_source
 ##
