@@ -15,8 +15,8 @@ from multiprocessing import dummy as mp_dummy
 from multiprocessing import get_context
 from operator import add, floordiv, mul, sub
 from pprint import pprint
-from unittest.mock import MagicMock
 from typing import Tuple
+from unittest.mock import MagicMock
 
 import pytest
 
@@ -36,7 +36,7 @@ from graphtik import (
     operations_reschedullled,
     tasks_marshalled,
 )
-from graphtik.modifiers import optional, sideffect, sol_sideffect, vararg
+from graphtik.modifiers import optional, sideffect, sideffected, vararg
 from graphtik.netop import NetworkOperation
 from graphtik.network import Solution
 from graphtik.op import Operation
@@ -136,8 +136,7 @@ def test_smoke_test():
     # Sum operation, late-bind compute function
     sum_op1 = operation(name="sum_op1", needs=["a", "b"], provides="sum_ab")(add)
 
-    # sum_op1 is callable
-    assert sum_op1(1, 2) == 3
+    assert sum_op1.fn(1, 2) == 3
 
     # Multiply operation, decorate in-place
     @operation(name="mul_op1", needs=["sum_ab", "b"], provides="sum_ab_times_b")
@@ -145,7 +144,7 @@ def test_smoke_test():
         return a * b
 
     # mul_op1 is callable
-    assert mul_op1(1, 2) == 2
+    assert mul_op1.fn(1, 2) == 2
 
     # Pow operation
     @operation(
@@ -171,9 +170,6 @@ def test_smoke_test():
         name="sum_op3", needs=["a", "b"], provides="sum_ab2"
     )
 
-    # sum_op3 is callable
-    assert sum_op3(5, 6) == 11
-
     # compose network
     netop = compose("my network", sum_op1, mul_op1, pow_op1, sum_op2, sum_op3)
 
@@ -369,29 +365,25 @@ def test_network_deep_merge():
     )
 
 
-def test_network_merge_in_doctests():
-    graphop = compose(
-        "graphop",
-        operation(name="mul1", needs=["a", "b"], provides=["ab"])(mul),
-        operation(name="sub1", needs=["a", "ab"], provides=["a_minus_ab"])(sub),
-        operation(
-            name="abspow1", needs=["a_minus_ab"], provides=["abs_a_minus_ab_cubed"]
-        )(partial(abspow, p=3)),
-    )
+def test_network_merge_in_doctests(merge_pipes):
+    graphop, another_graph = merge_pipes
+    merged_graph = compose("merged_graph", graphop, another_graph, merge=False)
 
-    another_graph = compose(
-        "another_graph",
-        operation(name="mul1", needs=["a", "b"], provides=["ab"])(mul),
-        operation(name="mul2", needs=["c", "ab"], provides=["cab"])(mul),
-    )
-    merged_graph = compose("merged_graph", graphop, another_graph, merge=True)
-    assert merged_graph.needs
-    assert merged_graph.provides
-
-    assert repr(merged_graph).startswith(
+    print(merged_graph)
+    assert str(merged_graph).startswith(
         "NetworkOperation('merged_graph', "
         "needs=['a', 'b', 'ab', 'a_minus_ab', 'c'], "
-        "provides=['ab', 'a_minus_ab', 'abs_a_minus_ab_cubed', 'cab'], x4 ops"
+        "provides=['ab', 'a_minus_ab', 'abs_a_minus_ab_cubed', 'cab'], x5 ops: "
+        "graphop.mul1, graphop.sub1, graphop.abspow1, another_graph.mul1, another_graph.mul2)"
+    )
+
+    merged_graph = compose("merged_graph", graphop, another_graph, merge=True)
+    print(merged_graph)
+    assert str(merged_graph).startswith(
+        "NetworkOperation('merged_graph', "
+        "needs=['a', 'b', 'ab', 'a_minus_ab', 'c'], "
+        "provides=['ab', 'a_minus_ab', 'abs_a_minus_ab_cubed', 'cab'], x4 ops: "
+        "mul1, sub1, abspow1, mul2)"
     )
 
 
@@ -865,7 +857,7 @@ def _box_increment(box):
 
 
 @pytest.fixture(params=[0, 1])
-def netop_sideffect1(request) -> NetworkOperation:
+def netop_sideffect1(request, exemethod) -> NetworkOperation:
     ops = [
         operation(
             name="extend", needs=["box", sideffect("a")], provides=[sideffect("b")]
@@ -877,17 +869,17 @@ def netop_sideffect1(request) -> NetworkOperation:
     if request.param:
         ops = reversed(ops)
     # Designate `a`, `b` as sideffect inp/out arguments.
-    graph = compose("sideffect1", *ops)
+    graph = compose("sideffect1", *ops, parallel=exemethod)
 
     return graph
 
 
-def test_sideffect_no_real_data(exemethod, netop_sideffect1: NetworkOperation):
+def test_sideffect_no_real_data(netop_sideffect1: NetworkOperation):
     sidefx_fail = is_marshal_tasks() and not isinstance(
         get_execution_pool(), types.FunctionType  # mp_dummy.Pool
     )
 
-    graph = netop_sideffect1.withset(parallel=exemethod)
+    graph = netop_sideffect1
     inp = {"box": [0], "a": True}
 
     ## Normal data must not match sideffects.
@@ -980,33 +972,123 @@ def test_sideffect_steps(exemethod, netop_sideffect1: NetworkOperation):
     assert "blue" in str(dot)
 
 
-def test_sideffect_NO_RESULT(caplog):
+def test_sideffect_NO_RESULT(caplog, exemethod):
+    # NO_RESULT does not cancel sideffects unless op-rescheduled
+    #
     sfx = sideffect("b")
-    op = operation(lambda: NO_RESULT, provides=sfx)
-    netop = compose("t", op)
+    op1 = operation(lambda: NO_RESULT, name="do-SFX", provides=sfx)
+    op2 = operation(lambda: 1, name="ask-SFX", needs=sfx, provides="a")
+    netop = compose("t", op1, op2, parallel=exemethod)
     sol = netop.compute({}, outputs=sfx)
+    assert op1 in sol.executed
+    assert op2 not in sol.executed
     assert sol == {}
-    assert op in sol.executed
+    sol = netop.compute({})
+    assert op1 in sol.executed
+    assert op2 in sol.executed
+    assert sol == {"a": 1}
+    sol = netop.compute({}, outputs="a")
+    assert op1 in sol.executed
+    assert op2 in sol.executed
+    assert sol == {"a": 1}
+
+    # NO_RESULT cancels sideffects of rescheduled ops.
+    #
+    netop = compose("t", op1, op2, rescheduled=True, parallel=exemethod)
+    sol = netop.compute({})
+    assert op1 in sol.executed
+    assert op2 not in sol.executed
+    assert sol == {sfx: False}
+    sol = netop.compute({}, outputs="a")
+    assert op2 not in sol.executed
+    assert op1 in sol.executed
+    assert sol == {}  # sfx evicted
 
     ## If NO_RESULT were not translated,
     #  a warning of unknown out might have emerged.
     caplog.clear()
-    netop = compose("t", operation(lambda: 1, provides=sfx))
+    netop = compose("t", operation(lambda: 1, provides=sfx), parallel=exemethod)
     netop.compute({}, outputs=sfx)
     for record in caplog.records:
         if record.levelname == "WARNING":
             assert "Ignoring result(1) because no `provides`" in record.message
 
     caplog.clear()
-    netop = compose("t", operation(lambda: NO_RESULT, provides=sfx))
+    netop = compose("t", operation(lambda: NO_RESULT, provides=sfx), parallel=exemethod)
     netop.compute({}, outputs=sfx)
     for record in caplog.records:
         assert record.levelname != "WARNING"
 
 
-@pytest.fixture
-def calc_prices_pipeline():
-    @operation(needs="order_items", provides=sol_sideffect("ORDER", "Items", "Prices"))
+def test_sideffect_cancel_sfx_only_operation(exemethod):
+    sfx = sideffect("b")
+    op1 = operation(
+        lambda: {sfx: False},
+        name="op1",
+        provides=sfx,
+        returns_dict=True,
+        rescheduled=True,
+    )
+    op2 = operation(lambda: 1, name="op2", needs=sfx, provides="a")
+    netop = compose("t", op1, op2, parallel=exemethod)
+    sol = netop.compute({})
+    assert sol == {sfx: False}
+    sol = netop.compute(outputs=sfx)
+    assert sol == {sfx: False}
+
+
+def test_sideffect_cancel(exemethod):
+    sfx = sideffect("b")
+    op1 = operation(
+        lambda: {"a": 1, sfx: False},
+        name="op1",
+        provides=["a", sfx],
+        returns_dict=True,
+        rescheduled=True,
+    )
+    op2 = operation(lambda: 1, name="op2", needs=sfx, provides="b")
+    netop = compose("t", op1, op2, parallel=exemethod)
+    sol = netop.compute()
+    assert sol == {"a": 1, sfx: False}
+    sol = netop.compute(outputs="a")
+    assert sol == {"a": 1}  # sfx evicted
+    ## SFX both pruned & evicted
+    #
+    assert sfx not in sol.dag.nodes
+    assert sfx in sol.plan.steps
+
+
+def test_sideffect_not_canceled_if_not_resched(exemethod):
+    # Check op without any provides
+    #
+    sfx = sideffect("b")
+    op1 = operation(lambda: {sfx: False}, name="op1", provides=sfx, returns_dict=True)
+    op2 = operation(lambda: 1, name="op2", needs=sfx, provides="b")
+    netop = compose("t", op1, op2, parallel=exemethod)
+    # sol = netop.compute()
+    # assert sol == {sfx: False, "b": 1}
+    sol = netop.compute(outputs="b")
+    assert sol == {"b": 1}
+
+    # Check also op with some provides
+    #
+    sfx = sideffect("b")
+    op1 = operation(
+        lambda: {"a": 1, sfx: False}, name="op1", provides=["a", sfx], returns_dict=True
+    )
+    op2 = operation(lambda: 1, name="op2", needs=sfx, provides="b")
+    netop = compose("t", op1, op2, parallel=exemethod)
+    sol = netop.compute()
+    assert sol == {"a": 1, sfx: False, "b": 1}
+    sol = netop.compute(outputs="b")
+    assert sol == {"b": 1}
+
+
+@pytest.fixture(params=[0, 1])
+def calc_prices_pipeline(request, exemethod):
+    """A pipeline that may work even without VAT-rates."""
+
+    @operation(needs="order_items", provides=sideffected("ORDER", "Items", "Prices"))
     def new_order(items: list) -> "pd.DataFrame":
         order = {"items": items}
         # Pretend we get the prices from sales.
@@ -1014,8 +1096,8 @@ def calc_prices_pipeline():
         return order
 
     @operation(
-        needs=[sol_sideffect("ORDER", "Items"), "vat rate"],
-        provides=sol_sideffect("ORDER", "VAT rates"),
+        needs=[sideffected("ORDER", "Items"), "vat rate"],
+        provides=sideffected("ORDER", "VAT rates"),
     )
     def fill_in_vat_ratios(order: "pd.DataFrame", base_vat: float) -> "pd.DataFrame":
         order["VAT_rates"] = [
@@ -1024,8 +1106,11 @@ def calc_prices_pipeline():
         return order
 
     @operation(
-        needs=[sol_sideffect("ORDER", "Prices"), sol_sideffect("ORDER", "VAT rates")],
-        provides=[sol_sideffect("ORDER", "VAT", "Totals"), "vat owed"],
+        needs=[
+            sideffected("ORDER", "Prices"),
+            sideffected("ORDER", "VAT rates", optional=True),
+        ],
+        provides=[sideffected("ORDER", "VAT", "Totals"), "vat owed"],
     )
     def finalize_prices(order: "pd.DataFrame") -> Tuple["pd.DataFrame", float]:
         if "VAT_rates" in order:
@@ -1034,18 +1119,18 @@ def calc_prices_pipeline():
             vat_to_pay = sum(order["VAT"])
         else:
             order["totals"] = order["prices"][::]
+            vat_to_pay = None
         return order, vat_to_pay
 
-    proc_order = compose(
-        "process order", new_order, fill_in_vat_ratios, finalize_prices
-    )
-    return proc_order
+    ops = [new_order, fill_in_vat_ratios, finalize_prices]
+    if request.param:
+        ops = reversed(ops)
+    return compose("process order", *ops, parallel=exemethod)
 
 
-def test_solution_sideffects_ok(calc_prices_pipeline):
-    sol = calc_prices_pipeline.compute(
-        {"order_items": "milk babylino toilet-paper".split(), "vat rate": 0.18}
-    )
+def test_sideffecteds_ok(calc_prices_pipeline):
+    inp = {"order_items": "milk babylino toilet-paper".split(), "vat rate": 0.18}
+    sol = calc_prices_pipeline.compute(inp)
     print(sol)
     assert sol == {
         "order_items": ["milk", "babylino", "toilet-paper"],
@@ -1059,14 +1144,39 @@ def test_solution_sideffects_ok(calc_prices_pipeline):
         },
         "vat owed": 1.44,
     }
+    sol = calc_prices_pipeline.compute(
+        inp, [sideffected("ORDER", "VAT"), sideffected("ORDER", "Totals")]
+    )
+    print(sol)
+    assert sol == {
+        "ORDER": {
+            "items": ["milk", "babylino", "toilet-paper"],
+            "prices": [1, 2, 3],
+            "VAT_rates": [0.18, 0.36, 0.18],
+            "VAT": [0.18, 0.72, 0.54],
+            "totals": [1.18, 2.7199999999999998, 3.54],
+        },
+        # "vat owed": 1.44,
+    }
+
+    ## `vat owed` both pruned & evicted
+    #
+    assert "vat owed" not in sol.dag.nodes
+    assert "vat owed" in sol.plan.steps
+    # Check Pruned+Evicted data plot as expected.
+    #
+    dot = str(sol.plot())
+    print(dot)
+    assert re.search(r"<vat owed>.+style=dashed", dot)
+    assert re.search(r'<vat owed>.+tooltip="\(evicted\)"', dot)
 
 
-def test_solution_sideffects_endured(calc_prices_pipeline):
+def test_sideffecteds_endured(calc_prices_pipeline):
     ## Break `fill_in_vat_ratios()`.
     #
     @operation(
-        needs=[sol_sideffect("ORDER", "Items"), "vat rate"],
-        provides=sol_sideffect("ORDER", "VAT rates"),
+        needs=[sideffected("ORDER", "Items"), "vat rate"],
+        provides=sideffected("ORDER", "VAT rates"),
         endured=True,
     )
     def fill_in_vat_ratios(order: "pd.DataFrame", base_vat: float) -> "pd.DataFrame":
@@ -1079,14 +1189,45 @@ def test_solution_sideffects_endured(calc_prices_pipeline):
     sol = calc_prices_pipeline.compute(
         {"order_items": "milk babylino toilet-paper".split(), "vat rate": 0.18}
     )
+
     print(sol)
     assert sol == {
         "order_items": ["milk", "babylino", "toilet-paper"],
         "vat rate": 0.18,
-        "ORDER": {"items": ["milk", "babylino", "toilet-paper"], "prices": [1, 2, 3]},
+        "ORDER": {
+            "items": ["milk", "babylino", "toilet-paper"],
+            "prices": [1, 2, 3],
+            "totals": [1, 2, 3],
+        },
+        "vat owed": None,
     }
 
 
+@pytest.fixture(params=[0, 1])
+def sideffected_resched(request, exemethod):
+    @operation(provides=sideffected("DEP", "yes", "no"), rescheduled=1, returns_dict=1)
+    def half_sfx():
+        return {"DEP": 1, sideffected("DEP", "no"): False}
+
+    yes = operation(
+        lambda dep: "yes!", name="YES", needs=sideffected("DEP", "yes"), provides="yes",
+    )
+    no = operation(
+        lambda dep: "no!", name="NO", needs=sideffected("DEP", "no"), provides="no",
+    )
+    ops = [half_sfx, yes, no]
+    if request.param:
+        ops = reversed(ops)
+    return compose("process order", *ops, parallel=exemethod)
+
+
+def test_sideffected_canceled(sideffected_resched):
+    """Check if a `returns-dict` op can cancel sideffecteds. """
+    sol = sideffected_resched.compute({})
+    print(sol)
+    assert sol == {"DEP": 1, sideffected("DEP", "no"): False, "yes": "yes!"}
+
+
 def test_optional_per_function_with_same_output(exemethod):
     # Test that the same need can be both optional and not on different operations.
     #
@@ -1206,7 +1347,7 @@ def test_skip_eviction_flag():
 
 
 @pytest.mark.parametrize(
-    "endurance, endured", [(None, True), (True, None), (False, True), (1, 1)]
+    "endurance, endured", [(None, True), (True, None), (1, 0), (1, 1)],
 )
 def test_execution_endurance(exemethod, endurance, endured):
     with operations_endured(endurance):
@@ -1248,7 +1389,7 @@ def test_execution_endurance(exemethod, endurance, endured):
 
 
 @pytest.mark.parametrize(
-    "resched, rescheduled", [(None, True), (True, None), (None, True), (1, 1)]
+    "resched, rescheduled", [(None, True), (True, None), (1, 0), (1, 1)],
 )
 def test_rescheduling(exemethod, resched, rescheduled):
     canc = operation(lambda: None, name="canc", needs=["b"], provides="cc")
@@ -1256,16 +1397,22 @@ def test_rescheduling(exemethod, resched, rescheduled):
         "netop",
         operation(lambda: [1], name="op1", provides=["a", "b"], rescheduled=1),
         canc,
-        operation(lambda: NO_RESULT, name="op2", provides=["c"], rescheduled=1),
+        operation(
+            lambda C=1: C and NO_RESULT,
+            name="op2",
+            needs=optional("C"),
+            provides=["c"],
+            rescheduled=1,
+        ),
         operation(
             lambda *args: sum(args),
             name="op3",
-            needs=["a", optional("b"), optional("c")],
+            needs=["a", vararg("b"), vararg("c")],
             provides=["d"],
         ),
         parallel=exemethod,
     )
-    sol = op()
+    sol = op.compute({})
     assert sol == {"a": 1, "d": 1}
     assert list(sol.canceled) == [canc]
     dot = str(sol.plot())
@@ -1276,7 +1423,13 @@ def test_rescheduling(exemethod, resched, rescheduled):
         assert sol.scream_if_incomplete()
 
     ## Check if modified state fails the 2nd time.
-    assert op() == {"a": 1, "d": 1}
+    assert op.compute({}) == {"a": 1, "d": 1}
+
+    ## Tell op to cancel just 1 of 2 provides
+    #  (the 2n one, 'b').
+    #
+    sol = op.compute({"C": False})
+    assert sol == {"C": False, "a": 1, "c": False, "d": 1}
 
 
 def test_rescheduling_NO_RESULT(exemethod):
