@@ -4,12 +4,19 @@
 import logging
 import re
 from collections import OrderedDict, namedtuple
+from functools import partial
 from types import SimpleNamespace
 
 import dill
 import pytest
 
 from graphtik import NO_RESULT, compose, operation, optional, sideffect, vararg, varargs
+from graphtik.config import (
+    operations_endured,
+    operations_reschedullled,
+    tasks_in_parallel,
+    tasks_marshalled,
+)
 from graphtik.network import yield_ops
 from graphtik.op import (
     FunctionalOperation,
@@ -54,6 +61,11 @@ def test_repr_returns_dict():
     )
 
 
+def test_auto_func_name():
+    assert operation(lambda: None).name == "<lambda>"
+    assert operation(partial(lambda a: None)).name == "<lambda>(...)"
+
+
 def test_builder_pattern():
     opb = operation()
     assert not isinstance(opb, FunctionalOperation)
@@ -106,14 +118,14 @@ def test_builder_pattern():
         (((), (), None), ((), (), ())),
         (((), [], None), ((), [], ())),
         (("", object(), None), ValueError("Cannot tuple-ize needs")),
-        (("", [None], None), ValueError("All `needs` must be str")),
-        (("", [()], None), ValueError("All `needs` must be str")),
+        (("", [None], None), TypeError("All `needs` must be str")),
+        (("", [()], None), TypeError("All `needs` must be str")),
         ## Check provides
         (((), "a", ()), ((), ("a",), ())),
         (((), "a", []), ((), ("a",), [])),
         (("", "a", object()), ValueError("Cannot tuple-ize provides")),
-        (("", "a", (None,)), ValueError("All `provides` must be str")),
-        (("", "a", [()]), ValueError("All `provides` must be str")),
+        (("", "a", (None,)), TypeError("All `provides` must be str")),
+        (("", "a", [()]), TypeError("All `provides` must be str")),
     ],
 )
 def test_func_op_validation(opargs, exp):
@@ -131,17 +143,17 @@ def test_func_op_validation(opargs, exp):
         (
             (),
             {"node_props": 3.14},
-            ValueError("node_props` must be a dict, was 'float':"),
+            TypeError("node_props` must be a dict, was 'float':"),
         ),
         (
             (),
             {"node_props": "ab"},
-            ValueError("node_props` must be a dict, was 'str':"),
+            TypeError("node_props` must be a dict, was 'str':"),
         ),
         ((), {"parents": []}, None),
-        ((), {"parents": ["gg"]}, ValueError("parents` must be tuple, was 'list':")),
-        ((), {"parents": 3.14}, ValueError("parents` must be tuple, was 'float':")),
-        ((), {"parents": "gg"}, ValueError("parents` must be tuple, was 'str':")),
+        ((), {"parents": ["gg"]}, TypeError("parents` must be tuple, was 'list':")),
+        ((), {"parents": 3.14}, TypeError("parents` must be tuple, was 'float':")),
+        ((), {"parents": "gg"}, TypeError("parents` must be tuple, was 'str':")),
     ],
 )
 def test_func_op_init(args, kw, exp):
@@ -205,7 +217,7 @@ def test_results_sequence_lt_many_provides(result, nfewer, asked_outputs):
 @pytest.mark.parametrize("result", ["", "a", "ab", "foobar", 3.14, None])
 def test_results_validation_bad_iterable(result, asked_outputs):
     op = operation(lambda: result, provides=["a", "b"])
-    with pytest.raises(ValueError, match=f"Expected x2 ITERABLE results, got"):
+    with pytest.raises(TypeError, match=f"Expected x2 ITERABLE results, got"):
         op.compute({}, outputs=asked_outputs)
 
 
@@ -255,7 +267,7 @@ def test_varargs():
 
 
 def test_op_node_props_bad():
-    with pytest.raises(ValueError, match="`node_props` must be"):
+    with pytest.raises(TypeError, match="`node_props` must be"):
         operation(lambda: None, name="a", node_props="SHOULD BE DICT")
 
 
@@ -332,24 +344,27 @@ def test_as_renames(inp, exp):
     "op_kw, ex",
     [
         (
-            dict(aliases={"a": 1}),
+            lambda: dict(aliases={"a": 1}),
             r"The `aliases` for ['a'] rename ['a'], not found in provides []!",
         ),
         (
-            dict(name="t", provides="a", aliases={"a": 1, "b": 2}),
+            lambda: dict(name="t", provides="a", aliases={"a": 1, "b": 2}),
             r"The `aliases` for ['a', 'b'] rename ['b'], not found in provides ['a']!",
         ),
         (
-            dict(name="t", provides=sideffect("a"), aliases={sideffect("a"): 1}),
+            lambda: dict(
+                name="t", provides=sideffect("a"), aliases={sideffect("a"): 1}
+            ),
             "must not contain `sideffects",
         ),
         (
-            dict(name="t", provides="a", aliases={"a": sideffect("AA")}),
+            lambda: dict(name="t", provides="a", aliases={"a": sideffect("AA")}),
             "must not contain `sideffects",
         ),
     ],
 )
 def test_provides_aliases_BAD(op_kw, ex):
+    op_kw = op_kw()
     with pytest.raises(ValueError, match=re.escape(ex)):
         operation(str, **op_kw)()
 
@@ -418,22 +433,35 @@ def test_parallel_op_repr():
 def test_marshalled_op_repr():
     op = operation(str, name="t", provides=["a"], marshalled=True)
     assert str(op) == (
-        "FunctionalOperation$(name='t', needs=[], provides=['a'], fn='str')"
+        "FunctionalOperation&(name='t', needs=[], provides=['a'], fn='str')"
     )
 
 
 def test_marshalled_parallel_op_repr():
     op = operation(str, name="t", parallel=1, marshalled=1)
-    assert str(op) == "FunctionalOperation|$(name='t', needs=[], provides=[], fn='str')"
+    assert str(op) == "FunctionalOperation|&(name='t', needs=[], provides=[], fn='str')"
 
 
 def test_ALL_op_repr():
     op = operation(str, name="t", rescheduled=1, endured=1, parallel=1, marshalled=1)
     assert (
-        str(op) == "FunctionalOperation!?|$(name='t', needs=[], provides=[], fn='str')"
+        str(op) == "FunctionalOperation!?|&(name='t', needs=[], provides=[], fn='str')"
     )
 
 
+def test_CONFIG_op_repr():
+    with operations_endured(1), operations_reschedullled(1), tasks_in_parallel(
+        1
+    ), tasks_marshalled(1):
+        op = operation(str, name="t")
+        assert (
+            str(op)
+            == "FunctionalOperation!?|&(name='t', needs=[], provides=[], fn='str')"
+        )
+    op = operation(str, name="t", rescheduled=0, endured=0, parallel=0, marshalled=0)
+    assert str(op) == "FunctionalOperation(name='t', needs=[], provides=[], fn='str')"
+
+
 def test_reschedule_outputs():
     op = operation(
         lambda: ["A", "B"], name="t", provides=["a", "b", "c"], rescheduled=True
