@@ -9,13 +9,18 @@ from operator import add
 
 import dill
 import networkx as nx
+import pydot
 import pytest
+from jinja2 import Template
 
 from graphtik import base, compose, network, operation, plot
+from graphtik.base import PlotArgs
 from graphtik.modifiers import optional
 from graphtik.netop import NetworkOperation
 from graphtik.plot import (
     Plotter,
+    Ref,
+    StylesStack,
     Theme,
     active_plotter_plugged,
     get_active_plotter,
@@ -98,12 +103,12 @@ def test_op_label_template_full():
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="&lt;op &quot; &#9; tooltip&gt;" HREF="http://op_url.com_label_" TARGET="_self"
                 ><FONT COLOR="blue"><B>OP:</B> <I>the op</I></FONT></TD>
                 <TD BORDER="1" SIDES="b"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
-                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured(!)" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>E</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled(?)" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>R</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel(|)" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>P</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled($)" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>M</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict({})" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>D</B></FONT></TD></TR>
+                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>!</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>?</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>|</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>&amp;</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>}</B></FONT></TD></TR>
                     </TABLE></TD>
             </TR><TR>
                 <TD COLSPAN="2" ALIGN="left" TOOLTIP="&lt;fn&#10;tooltip&gt;" HREF="http://fn_url.com/quoto_and" TARGET="_top"
@@ -351,20 +356,37 @@ def test_style_Ref():
     class C:
         arch_url = "1"
 
-    s = s.resolve(C)
-    assert str(s) == "1"
+    class C2:
+        arch_url = "2"
+
+    ss = s.resolve(C)
+    assert str(ss) == "1"
+    ss = s.resolve(C, C2)
+    assert str(ss) == "1"
+    ss = s.resolve(C2, C)
+    assert str(ss) == "2"
 
     r = plot.Ref("resched_thickness")  # int target
     str(r)  # should not scream
     assert r.resolve(Theme) == 4
 
+    ## Test default
+    s = plot.Ref("bad")
+    ss = s.resolve(C, C2, default=4)
+    assert ss == 4
+    s = plot.Ref("bad", default=5)
+    ss = s.resolve(C, C2, default=4)
+    assert ss == 4
+    ss = s.resolve(C, C2)
+    assert ss == 5
 
-def test_Theme_with_set():
+
+def test_Theme_withset():
     p = Theme()
-    p2 = p.with_set()
+    p2 = p.withset()
     assert p is not p2
 
-    p3 = p.with_set(include_steps=True)
+    p3 = p.withset(include_steps=True)
     assert p3.include_steps
     assert not p.include_steps
 
@@ -380,6 +402,56 @@ def test_Plotter_with_styles():
     assert not p.default_theme.include_steps
 
 
+def test_StylesStack_expansion_ok():
+    plot_args = PlotArgs(
+        name="James",
+        theme=Theme(
+            surname="Bond",
+            test_style1={"a_list": [{"ref": Ref("surname")}]},
+            test_style2={
+                "a_list": [{"callable": lambda plot_args: f"{plot_args.name}"}]
+            },
+            test_style3={"a_list": [Template("{{ name }} {{theme.surname}}")]},
+        ),
+    )
+    styles = StylesStack(plot_args, [])
+    styles.add("test_style1")
+    styles.add("test_style2")
+    styles.add("test_style3")
+    kw = styles.merge()
+
+    print(kw["a_list"])
+    assert kw["a_list"] == [
+        {"ref": "Bond"},
+        {"callable": "James"},
+        "James Bond",
+    ]
+
+
+def test_StylesStack_expansion_skip_item():
+    plot_args = PlotArgs(theme=Theme(test_style={"foo": lambda pa: ...}))
+    styles = StylesStack(plot_args, [])
+    styles.add("test_style")
+    kw = styles.merge()
+    assert kw == {}
+
+
+@pytest.mark.parametrize(
+    "style, exp",
+    [
+        (Ref("BAD_REF"), "BAD_REF"),
+        (Template("{{ BAD_VAR.WORSE }}"), "BAD_VAR"),
+        (lambda pa: BAD_CALL, "BAD_CALL"),  # pylint: disable=undefined-variable
+    ],
+)
+def test_StylesStack_expansion_BAD(style, exp):
+    plot_args = PlotArgs(theme=Theme(test_style={"foo": style}))
+    styles = StylesStack(plot_args, [])
+    styles.add("test_style")
+    with pytest.raises(ValueError, match=exp):
+        styles.merge()
+
+
 def test_plotter_customizations(pipeline, monkeypatch):
     ## default URL
     #
@@ -471,26 +543,26 @@ def test_node_dot_str0(dot_str_pipeline):
         digraph graph_ {
         fontname=italic;
         label=<graph>;
-        <edge> [margin="0.02,0.02", shape=invhouse];
-        <digraph&#58; strict> [margin="0.02,0.02", shape=invhouse];
+        <edge> [margin="0.04,0.02", shape=invhouse];
+        <digraph&#58; strict> [margin="0.04,0.02", shape=invhouse];
         <node> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET=""
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>node</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD COLSPAN="2" ALIGN="left" TOOLTIP="Same as a + b." TARGET=""
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="Same as a + b." TARGET="_top"
                 ><B>FN:</B> &lt;built-in function add&gt;</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<node>];
-        <&lt;graph&gt;> [margin="0.02,0.02", shape=house];
+        <&lt;graph&gt;> [margin="0.04,0.02", shape=house];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET=""
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET=""
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET="_top"
                 ><B>FN:</B> test.test_plot.func</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
@@ -500,7 +572,7 @@ def test_node_dot_str0(dot_str_pipeline):
         <digraph&#58; strict> -> <cu&#58;sto&#58;m>;
         <node> -> <&lt;graph&gt;>;
         <cu&#58;sto&#58;m> -> <&lt;graph&gt;>;
-        legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_top];
+        legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_blank];
         }
         """
 
@@ -515,25 +587,25 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
     #
     overlay = nx.DiGraph()
     hidden_op = dot_str_pipeline.net.find_op_by_name("node")
-    overlay.add_node(hidden_op, _no_plot=True)
-    overlay.graph["splines"] = "ortho"
+    overlay.add_node(hidden_op, no_plot=True)
+    overlay.graph["graphviz.splines"] = "ortho"
 
     exp = """
         digraph solution_x5_nodes {
         fontname=italic;
         splines=ortho;
         subgraph "cluster_after pruning" {
-        label="after pruning";
-        <edge> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
-        <digraph&#58; strict> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
-        <&lt;graph&gt;> [fillcolor=SkyBlue, margin="0.02,0.02", shape=house, style=filled, tooltip="(None)"];
+        label=<after pruning>;
+        <edge> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+        <digraph&#58; strict> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
+        <&lt;graph&gt;> [fillcolor=SkyBlue, margin="0.04,0.02", shape=house, style=filled, tooltip="(None)"];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" HREF="abc#{&#x27;dot_path&#x27;: &#x27;test.test_plot.func&#x27;, &#x27;posix_path&#x27;: &#x27;test/test_plot/func&#x27;}" TARGET="bad"
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" HREF="abc#{&#x27;dot_path&#x27;: &#x27;test.test_plot.func&#x27;, &#x27;posix_path&#x27;: &#x27;test/test_plot/func&#x27;}" TARGET="bad"
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET="_top"
                 ><B>FN:</B> test.test_plot.func</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
@@ -542,7 +614,7 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
         <edge> -> <cu&#58;sto&#58;m>;
         <digraph&#58; strict> -> <cu&#58;sto&#58;m>;
         <cu&#58;sto&#58;m> -> <&lt;graph&gt;>;
-        legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_top];
+        legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_blank];
         }
         """
 
@@ -551,12 +623,7 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
     sol = dot_str_pipeline.compute({"edge": 1, "digraph: strict": 2})
     dot_str = str(
         sol.plot(
-            graph=overlay,
-            theme=Theme(
-                py_item_url_format="abc#%s",
-                op_link_target="_self",
-                fn_link_target="bad",
-            ),
+            graph=overlay, theme=Theme(op_link_target="_self", fn_link_target="bad",),
         )
     )
 
@@ -570,20 +637,17 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
         sol.plot(
             graph=overlay,
             plotter=get_active_plotter().with_styles(
-                py_item_url_format="abc#%s",
-                op_link_target="_self",
-                fn_link_target="bad",
+                op_link_target="_self", fn_link_target="bad",
             ),
         )
     )
     assert _striplines(dot_str) == _striplines(exp)
     assert "<node>" not in dot_str
 
-    ## (last one) Patch theme directly on default active-plotter.
-    #
-    theme = get_active_plotter().default_theme
-    monkeypatch.setattr(theme, "py_item_url_format", "abc#%s")
-    monkeypatch.setattr(theme, "op_link_target", "_self")
-    monkeypatch.setattr(theme, "fn_link_target", "bad")
-    assert _striplines(dot_str) == _striplines(exp)
-    assert "<node>" not in dot_str
+
+def test_combine_clusters(merge_pipes):
+    p1, p2 = merge_pipes
+    merged_graph = compose("m", p1, p2, merge=False)
+    dot: pydot.Dot = merged_graph.plot()
+    assert dot.get_subgraph(f"cluster_{p1.name}")
+    assert dot.get_subgraph(f"cluster_{p2.name}")
